package java.time.chrono;

import java.time.temporal.Temporal;
import java.io.ObjectInput;
import java.io.IOException;
import java.io.ObjectOutput;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.LocalTime;
import java.time.temporal.TemporalUnit;
import java.time.temporal.TemporalAmount;
import java.time.temporal.TemporalAdjuster;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.ChronoField;
import java.time.temporal.ValueRange;
import java.time.temporal.TemporalField;
import java.time.temporal.TemporalAccessor;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.Clock;
import java.io.Serializable;

public final class HijrahDate extends ChronoLocalDateImpl<HijrahDate> implements ChronoLocalDate, Serializable
{
    private static final long serialVersionUID = -5207853542612002020L;
    private final transient HijrahChronology chrono;
    private final transient int prolepticYear;
    private final transient int monthOfYear;
    private final transient int dayOfMonth;
    
    static HijrahDate of(HijrahChronology hijrahChronology, int n, int n2, int n3) {
        return new HijrahDate(hijrahChronology, n, n2, n3);
    }
    
    static HijrahDate ofEpochDay(HijrahChronology hijrahChronology, long n) {
        return new HijrahDate(hijrahChronology, n);
    }
    
    public static HijrahDate now() {
        return now(Clock.systemDefaultZone());
    }
    
    public static HijrahDate now(ZoneId zone) {
        return now(Clock.system(zone));
    }
    
    public static HijrahDate now(Clock clock) {
        return ofEpochDay(HijrahChronology.INSTANCE, LocalDate.now(clock).toEpochDay());
    }
    
    public static HijrahDate of(int n, int n2, int n3) {
        return HijrahChronology.INSTANCE.date(n, n2, n3);
    }
    
    public static HijrahDate from(TemporalAccessor temporalAccessor) {
        return HijrahChronology.INSTANCE.date(temporalAccessor);
    }
    
    private HijrahDate(HijrahChronology chrono, int prolepticYear, int monthOfYear, int dayOfMonth) {
        chrono.getEpochDay(prolepticYear, monthOfYear, dayOfMonth);
        this.chrono = chrono;
        this.prolepticYear = prolepticYear;
        this.monthOfYear = monthOfYear;
        this.dayOfMonth = dayOfMonth;
    }
    
    private HijrahDate(HijrahChronology chrono, long n) {
        int[] hijrahDateInfo = chrono.getHijrahDateInfo((int)n);
        this.chrono = chrono;
        this.prolepticYear = hijrahDateInfo[0];
        this.monthOfYear = hijrahDateInfo[1];
        this.dayOfMonth = hijrahDateInfo[2];
    }
    
    @Override
    public HijrahChronology getChronology() {
        return this.chrono;
    }
    
    @Override
    public HijrahEra getEra() {
        return HijrahEra.AH;
    }
    
    @Override
    public int lengthOfMonth() {
        return this.chrono.getMonthLength(this.prolepticYear, this.monthOfYear);
    }
    
    @Override
    public int lengthOfYear() {
        return this.chrono.getYearLength(this.prolepticYear);
    }
    
    @Override
    public ValueRange range(TemporalField obj) {
        if ((obj instanceof ChronoField)) {
            if (this.isSupported(obj)) {
                ChronoField chronoField = (ChronoField)obj;
                switch (chronoField) {
                    case DAY_OF_MONTH: {
                        return ValueRange.of(1L, (long)this.lengthOfMonth());
                    }
                    case DAY_OF_YEAR: {
                        return ValueRange.of(1L, (long)this.lengthOfYear());
                    }
                    case ALIGNED_WEEK_OF_MONTH: {
                        return ValueRange.of(1L, 5L);
                    }
                    default: {
                        return this.getChronology().range(chronoField);
                    }
                }
            }
            else {
                throw new UnsupportedTemporalTypeException("Unsupported field: " + obj);
            }
        }
        else {
            return obj.rangeRefinedBy(this);
        }
    }
    
    @Override
    public long getLong(TemporalField obj) {
        if ((obj instanceof ChronoField)) {
            switch ((ChronoField)obj) {
                case DAY_OF_WEEK: {
                    return (long)this.getDayOfWeek();
                }
                case ALIGNED_DAY_OF_WEEK_IN_MONTH: {
                    return (long)((this.getDayOfWeek() - 1) % 7 + 1);
                }
                case ALIGNED_DAY_OF_WEEK_IN_YEAR: {
                    return (long)((this.getDayOfYear() - 1) % 7 + 1);
                }
                case DAY_OF_MONTH: {
                    return (long)this.dayOfMonth;
                }
                case DAY_OF_YEAR: {
                    return (long)this.getDayOfYear();
                }
                case EPOCH_DAY: {
                    return this.toEpochDay();
                }
                case ALIGNED_WEEK_OF_MONTH: {
                    return (long)((this.dayOfMonth - 1) / 7 + 1);
                }
                case ALIGNED_WEEK_OF_YEAR: {
                    return (long)((this.getDayOfYear() - 1) / 7 + 1);
                }
                case MONTH_OF_YEAR: {
                    return (long)this.monthOfYear;
                }
                case PROLEPTIC_MONTH: {
                    return this.getProlepticMonth();
                }
                case YEAR_OF_ERA: {
                    return (long)this.prolepticYear;
                }
                case YEAR: {
                    return (long)this.prolepticYear;
                }
                case ERA: {
                    return (long)this.getEraValue();
                }
                default: {
                    throw new UnsupportedTemporalTypeException("Unsupported field: " + obj);
                }
            }
        }
        else {
            return obj.getFrom(this);
        }
    }
    
    private long getProlepticMonth() {
        return (long)this.prolepticYear * 12L + (long)this.monthOfYear - 1L;
    }
    
    @Override
    public HijrahDate with(TemporalField obj, long value) {
        if ((obj instanceof ChronoField)) {
            ChronoField field = (ChronoField)obj;
            this.chrono.range(field).checkValidValue(value, field);
            int a = (int)value;
            switch (field) {
                case DAY_OF_WEEK: {
                    return this.plusDays(value - (long)this.getDayOfWeek());
                }
                case ALIGNED_DAY_OF_WEEK_IN_MONTH: {
                    return this.plusDays(value - this.getLong(ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH));
                }
                case ALIGNED_DAY_OF_WEEK_IN_YEAR: {
                    return this.plusDays(value - this.getLong(ChronoField.ALIGNED_DAY_OF_WEEK_IN_YEAR));
                }
                case DAY_OF_MONTH: {
                    return this.resolvePreviousValid(this.prolepticYear, this.monthOfYear, a);
                }
                case DAY_OF_YEAR: {
                    return this.plusDays((long)(Math.min(a, this.lengthOfYear()) - this.getDayOfYear()));
                }
                case EPOCH_DAY: {
                    return new HijrahDate(this.chrono, value);
                }
                case ALIGNED_WEEK_OF_MONTH: {
                    return this.plusDays((value - this.getLong(ChronoField.ALIGNED_WEEK_OF_MONTH)) * 7L);
                }
                case ALIGNED_WEEK_OF_YEAR: {
                    return this.plusDays((value - this.getLong(ChronoField.ALIGNED_WEEK_OF_YEAR)) * 7L);
                }
                case MONTH_OF_YEAR: {
                    return this.resolvePreviousValid(this.prolepticYear, a, this.dayOfMonth);
                }
                case PROLEPTIC_MONTH: {
                    return this.plusMonths(value - this.getProlepticMonth());
                }
                case YEAR_OF_ERA: {
                    return this.resolvePreviousValid((this.prolepticYear >= 1) ? a : (1 - a), this.monthOfYear, this.dayOfMonth);
                }
                case YEAR: {
                    return this.resolvePreviousValid(a, this.monthOfYear, this.dayOfMonth);
                }
                case ERA: {
                    return this.resolvePreviousValid(1 - this.prolepticYear, this.monthOfYear, this.dayOfMonth);
                }
                default: {
                    throw new UnsupportedTemporalTypeException("Unsupported field: " + obj);
                }
            }
        }
        else {
            return (HijrahDate)super.with(obj, value);
        }
    }
    
    private HijrahDate resolvePreviousValid(int n, int n2, int n3) {
        int monthLength = this.chrono.getMonthLength(n, n2);
        if (n3 > monthLength) {
            n3 = monthLength;
        }
        return of(this.chrono, n, n2, n3);
    }
    
    @Override
    public HijrahDate with(TemporalAdjuster temporalAdjuster) {
        return (HijrahDate)super.with(temporalAdjuster);
    }
    
    public HijrahDate withVariant(HijrahChronology hijrahChronology) {
        if (this.chrono == hijrahChronology) {
            return this;
        }
        int dayOfYear = hijrahChronology.getDayOfYear(this.prolepticYear, this.monthOfYear);
        return of(hijrahChronology, this.prolepticYear, this.monthOfYear, (this.dayOfMonth > dayOfYear) ? dayOfYear : this.dayOfMonth);
    }
    
    @Override
    public HijrahDate plus(TemporalAmount temporalAmount) {
        return (HijrahDate)super.plus(temporalAmount);
    }
    
    @Override
    public HijrahDate minus(TemporalAmount temporalAmount) {
        return (HijrahDate)super.minus(temporalAmount);
    }
    
    @Override
    public long toEpochDay() {
        return this.chrono.getEpochDay(this.prolepticYear, this.monthOfYear, this.dayOfMonth);
    }
    
    private int getDayOfYear() {
        return this.chrono.getDayOfYear(this.prolepticYear, this.monthOfYear) + this.dayOfMonth;
    }
    
    private int getDayOfWeek() {
        return (int)Math.floorMod(this.toEpochDay() + 3L, 7L) + 1;
    }
    
    private int getEraValue() {
        return this.prolepticYear > 1?1:0;
    }
    
    @Override
    public boolean isLeapYear() {
        return this.chrono.isLeapYear((long)this.prolepticYear);
    }
    
    @Override
    HijrahDate plusYears(long n) {
        if (n == 0L) {
            return this;
        }
        return this.resolvePreviousValid(Math.addExact(this.prolepticYear, (int)n), this.monthOfYear, this.dayOfMonth);
    }
    
    @Override
    HijrahDate plusMonths(long n) {
        if (n == 0L) {
            return this;
        }
        long n2 = (long)this.prolepticYear * 12L + (long)(this.monthOfYear - 1) + n;
        return this.resolvePreviousValid(this.chrono.checkValidYear(Math.floorDiv(n2, 12L)), (int)Math.floorMod(n2, 12L) + 1, this.dayOfMonth);
    }
    
    @Override
    HijrahDate plusWeeks(long n) {
        return (HijrahDate)super.plusWeeks(n);
    }
    
    @Override
    HijrahDate plusDays(long n) {
        return new HijrahDate(this.chrono, this.toEpochDay() + n);
    }
    
    @Override
    public HijrahDate plus(long n, TemporalUnit temporalUnit) {
        return (HijrahDate)super.plus(n, temporalUnit);
    }
    
    @Override
    public HijrahDate minus(long n, TemporalUnit temporalUnit) {
        return (HijrahDate)super.minus(n, temporalUnit);
    }
    
    @Override
    HijrahDate minusYears(long n) {
        return (HijrahDate)super.minusYears(n);
    }
    
    @Override
    HijrahDate minusMonths(long n) {
        return (HijrahDate)super.minusMonths(n);
    }
    
    @Override
    HijrahDate minusWeeks(long n) {
        return (HijrahDate)super.minusWeeks(n);
    }
    
    @Override
    HijrahDate minusDays(long n) {
        return (HijrahDate)super.minusDays(n);
    }
    
    @Override
    public final ChronoLocalDateTime<HijrahDate> atTime(LocalTime localTime) {
        return super.atTime(localTime);
    }
    
    @Override
    public ChronoPeriod until(ChronoLocalDate chronoLocalDate) {
        HijrahDate date = this.getChronology().date(chronoLocalDate);
        long n = (long)((date.prolepticYear - this.prolepticYear) * 12 + (date.monthOfYear - this.monthOfYear));
        int n2 = date.dayOfMonth - this.dayOfMonth;
        if (n > 0L && n2 < 0) {
            --n;
            n2 = (int)(date.toEpochDay() - this.plusMonths(n).toEpochDay());
        }
        else if (n < 0L && n2 > 0) {
            ++n;
            n2 -= date.lengthOfMonth();
        }
        return this.getChronology().period(Math.toIntExact(n / 12L), (int)(n % 12L), n2);
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if ((o instanceof HijrahDate)) {
            HijrahDate hijrahDate = (HijrahDate)o;
            return this.prolepticYear == hijrahDate.prolepticYear && this.monthOfYear == hijrahDate.monthOfYear && this.dayOfMonth == hijrahDate.dayOfMonth && this.getChronology().equals(hijrahDate.getChronology());
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        int prolepticYear = this.prolepticYear;
        return this.getChronology().getId().hashCode() ^ (prolepticYear & 0xFFFFF800) ^ (prolepticYear << 11) + (this.monthOfYear << 6) + this.dayOfMonth;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    private Object writeReplace() {
        return new Ser((byte)6, this);
    }
    
    void writeExternal(ObjectOutput objectOutput) throws IOException {
        objectOutput.writeObject(this.getChronology());
        objectOutput.writeInt(this.get(ChronoField.YEAR));
        objectOutput.writeByte(this.get(ChronoField.MONTH_OF_YEAR));
        objectOutput.writeByte(this.get(ChronoField.DAY_OF_MONTH));
    }
    
    static HijrahDate readExternal(ObjectInput objectInput) throws IOException, ClassNotFoundException {
        return ((HijrahChronology)objectInput.readObject()).date(objectInput.readInt(), (int)objectInput.readByte(), (int)objectInput.readByte());
    }
}
